package de.opalpiraten.scenepainter.gui;
import java.awt.*;  
import java.io.*;    
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import java.beans.*;
import java.net.*;
import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;

/**
 * The ProjectEditor is the GUI frontend for the Project class.
 */
public class ProjectEditor extends JInternalFrame implements ActionListener,
ListSelectionListener, WorkspaceListener {

  /**
   * The Project currently being worked on
   */
  private Project project;
  
  /**
   * Reference to the scene showing
   */
  private Scene scene;
  
  /**
   * The list of available scenes
   */
  private JList scenelist = new JList();
  
  /**
   * List of sceneelements
   */
  private JList elementlist = new JList();
  
  /**
   * Lsit of pictures
   */
  private JList picturelist = new JList();
  
  /**
   * Userinput: Save a sceneelement under this name
   */
  private JTextField saveelementas = new JTextField();
  
  /**
   * the OK Button for the saveas textfield
   */
  private JButton saveelement = new  JButton(Util.createIcon("toolbarButtonGraphics/general/Save16.gif",""));
  
  /**
   * Userinput: Save a scene under this name
   */
  private JTextField savesceneas = new JTextField();
  
  /**
   * the OK Button for the saveas textfield
   */
  private JButton savescene = new  JButton(Util.createIcon("toolbarButtonGraphics/general/Save16.gif",""));
  
  /**
   * List all available projects
   */
  private JComboBox plist = new JComboBox();
  
  /**
   * Switch between scenes and sceneelements. It is expected, that the first
   * tab contains the scene and the second the element list.
   */
  private JTabbedPane manage = new JTabbedPane(JTabbedPane.TOP);
  
  /**
   * For deleting a scene from the list
   */
  private JMenuItem deletescene = new JMenuItem(Dict.get("gui.projecteditor.delete"));

  /**
   * For deleting a sceneelement from the list
   */
  private JMenuItem deletesceneelement = new JMenuItem(Dict.get("gui.projecteditor.delete"));

  /**
   * Deleting a picture from the project
   */
  private JButton deletepicture = new JButton(Util.createIcon("toolbarButtonGraphics/general/Delete16.gif",""));
  
  /**
   * Import pictures to project
   */
  private JButton importpicture = new JButton(Util.createIcon("toolbarButtonGraphics/general/Import16.gif",""));
  
  /**
   * Export scenes as PNG?
   */
  private JCheckBox export = new JCheckBox(Dict.get("gui.projecteditor.export"),true);
  
  /**
   * Reload project
   */
  private JButton reload = new JButton(Util.createIcon("toolbarButtonGraphics/general/Refresh16.gif",""));
  
  /**
   * Stop replaceProject from becoming reentrant
   */
  private boolean noupdate=false;
  
  public ProjectEditor() {
    super(Dict.get("gui.projecteditor.title",""),false,false,false,true);
    scenelist.addListSelectionListener(this);
    elementlist.addListSelectionListener(this);
    savesceneas.addActionListener(this);
    savescene.addActionListener(this);
    saveelementas.addActionListener(this);
    saveelement.addActionListener(this);
    deletepicture.addActionListener(this);
    importpicture.addActionListener(this);
    plist.addActionListener(this);
    deletescene.addActionListener(this);
    deletesceneelement.addActionListener(this);
    reload.addActionListener(this);
    Menus.clear.addActionListener(this);
    
    JPopupMenu popscene = new JPopupMenu();
    popscene.add(deletescene);
    scenelist.setComponentPopupMenu(popscene);
    
    JPopupMenu popsceneelement = new JPopupMenu();
    popsceneelement.add(deletesceneelement);
    
    elementlist.setComponentPopupMenu(popsceneelement); 
    
    JPanel content =new JPanel();
    
     
    setContentPane(content);
    content.setLayout(new BoxLayout(content,BoxLayout.Y_AXIS));
    
    JPanel startload = new JPanel();
    startload.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.projecteditor.startorload")));
    
    
    startload.add(plist);
    startload.add(Box.createHorizontalStrut(10));
    startload.add(reload);
    plist.setEditable(true);
    startload.setLayout(new BoxLayout(startload,BoxLayout.X_AXIS));
    content.add(startload);
    
    
    manage.setBorder(BorderFactory.createTitledBorder(Dict.get("gui.projecteditor.manage.title")));
    
    JPanel mscenes = new JPanel();
    GridBagLayout gbl = new GridBagLayout();
    mscenes.setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=1.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(savesceneas,gbc);
    mscenes.add(savesceneas);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=1;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(savescene,gbc);
    mscenes.add(savescene);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=2; 
    gbc.gridheight=1; 
    gbc.weightx=0.1;
    gbc.weighty=0.1;
    gbc.insets = new Insets(1,1,1,1);
    JScrollPane jsp0 = new JScrollPane(scenelist);
    gbl.setConstraints(jsp0,gbc);
    mscenes.add(jsp0);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=2;
    gbc.gridwidth=2; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(export,gbc);
    mscenes.add(export);
    manage.addTab(Dict.get("gui.projecteditor.manage.scenes"),mscenes);
    

    JPanel melements = new JPanel();
    gbl = new GridBagLayout();
    melements.setLayout(gbl);
    gbc = new GridBagConstraints();
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=1.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(saveelementas,gbc);
    melements.add(saveelementas);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=1;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1; 
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(1,1,1,1);
    gbl.setConstraints(saveelement,gbc);
    melements.add(saveelement);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=2; 
    gbc.gridheight=1; 
    gbc.weightx=0.1;
    gbc.weighty=0.1;
    gbc.insets = new Insets(1,1,1,1);
    JScrollPane jsp1 = new JScrollPane(elementlist);
    gbl.setConstraints(jsp1,gbc);
    melements.add(jsp1);
    
    manage.addTab(Dict.get("gui.projecteditor.manage.elements"),melements);
    
    JPanel mpictures = new JPanel();
    mpictures.setLayout(new BoxLayout(mpictures,BoxLayout.Y_AXIS));
    JPanel submpictures = new JPanel();
    //submpictures.setLayout(new BoxLayout(submpictures,BoxLayout.X_AXIS));
    submpictures.add(importpicture);
    //submpictures.add(Box.createHorizontalStrut(5));
    submpictures.add(deletepicture);
    mpictures.add(new JScrollPane(picturelist));
    //mpictures.add(Box.createVerticalStrut(2));
    mpictures.add(submpictures);
    
    
    manage.addTab(Dict.get("gui.projecteditor.manage.pictures"),mpictures);
    
    
    content.add(manage);
    
    pack();
  }
  
  // Interface implemented
  public void replaceProject(Project p) {
    if (noupdate) return;
    noupdate=true;
    project =p;
    DefaultComboBoxModel dcbm = new DefaultComboBoxModel(Project.listProjects());
    plist.setModel(dcbm);
    plist.setSelectedItem(p.getName());
    
    try {
      scenelist.setListData(new Object[0]);
      scenelist.setListData(p.listScenes());
    
      elementlist.setListData(new Object[0]);
      elementlist.setListData(p.listSceneElements());
      
      picturelist.setListData(new Object[0]);
      picturelist.setListData(p.listPictures());
    }
    catch (Exception exp) {
      exp.printStackTrace();
    }
    noupdate=false;
  }
  
  // Interface implemented
  public void replaceScene(Scene s) {
    scene=s;
  }
  
  /**
   * Query project shown in the editor
   */
  public Project getProject() {
    return project;
  }
  
  /**
   * Handle saving
   */
  private void handleSaveScene() {
    try {
      if (scene==null || savesceneas.getText().length()==0) return;

      scene.setName(savesceneas.getText());
      project.saveScene(scene,export.isSelected());
      scenelist.setListData(project.listScenes());
      // Propagate
      Core.workspace.setScene(scene);
      Core.workspace.setProject(project);
    }
    catch (Exception exp) {
      exp.printStackTrace();
    }
  }
  
  private void handleSaveSceneElement() {
    try {
      if (scene==null || saveelementas.getText().length()==0) return;
      
      SceneElement se = scene.getSelected();
      if (se==null) return;
      se.setName(saveelementas.getText());
      project.saveSceneElement(se);
      saveelementas.setText("");
      Core.workspace.setProject(project);
    }
    catch (Exception exp) {
      exp.printStackTrace();
    }
  }
  
  /**
   * Handle loading or creation of projects
   */
  private void handleProject() {
    String name = plist.getSelectedItem().toString();
    if (name==null || name.equals("")) return; 
    Project p = new Project(name);
    Core.workspace.setProject(p);
  }
  
  /**
   * Handle delete requests for scenes
   */
  private void handleDeleteScene() {
    try {
      Object names[] = scenelist.getSelectedValues();
      for (int i=0;i<names.length;i++) project.deleteScene(names[i].toString());
      Core.workspace.setProject(project);
    }
    catch (Exception exp) {
      exp.printStackTrace();
    }
  }
  
  /**
   * Handle delete requests for sceneelements
   */
  private void handleDeleteSceneElement() {
    try {
      Object names[] = elementlist.getSelectedValues();
      for (int i=0;i<names.length;i++) project.deleteSceneElement(names[i].toString());
      Core.workspace.setProject(project);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
  
  /**
   * Handle deletion of pictures
   */
  private void handleDeletePicture() {
    try {
      Object names[] = picturelist.getSelectedValues();
      for (int i=0;i<names.length;i++) {
        project.getFileFor(Project.IMPORTDIR,names[i].toString()).delete();
      }
      Core.workspace.setProject(project);
    }
    catch (Exception exp) {
      exp.printStackTrace();
    }
  }
  
  /**
   * Handle picture import
   */
  private void handleImportPicture() {
    try {
      JFileChooser jfc = new JFileChooser();
      jfc.setMultiSelectionEnabled(true);
      jfc.setDialogTitle(Dict.get("gui.projecteditor.import"));  
      jfc.setFileFilter(new ChooserSuffixFilter(SuffixFilter.PNG));
      jfc.showDialog(Core.workspace,Dict.get("gui.projecteditor.import.approve")); 
      File f[] = jfc.getSelectedFiles();
      if (f!=null) project.importPicture(f);
      
      Core.workspace.setProject(project);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }
  
  /**
   * User requested a new Scene through the menu
   */
  private void handleClear() {
    if(scene !=null && scene.dirty) {
      if (JOptionPane.YES_OPTION != JOptionPane.showInternalOptionDialog(Core.workspace,Dict.get("gui.projecteditor.warn.load"),Dict.get("gui.projecteditor.warn.load.title"),JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,null,null)) {
        return;
      }
    }
    Core.workspace.setScene(Util.newScene());
  }
  
  // Interface implented
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    
    if (src==savesceneas || src==savescene) handleSaveScene();
    if (src==saveelementas || src==saveelement) handleSaveSceneElement();
    if (src==plist) handleProject();
    if (src==deletescene) handleDeleteScene();
    if (src==deletesceneelement) handleDeleteSceneElement();
    if (src==importpicture) handleImportPicture();
    if (src==deletepicture) handleDeletePicture();
    if (src==reload) Core.workspace.setProject(project);
    if (src==Menus.clear) handleClear();
  }
  
  // Interface implemented
  public void valueChanged(ListSelectionEvent e) {
    if (e.getSource()==scenelist) {
      if (scenelist.getSelectedIndex()<0 || e.getValueIsAdjusting()) return;
      if(scene !=null && scene.dirty) {
        if (JOptionPane.YES_OPTION != JOptionPane.showInternalOptionDialog(Core.workspace,Dict.get("gui.projecteditor.warn.load"),Dict.get("gui.projecteditor.warn.load.title"),JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE,null,null,null)) {
          return;
        }
      }
      try {
        scene = project.loadScene((String)scenelist.getSelectedValue());
        Core.workspace.setScene(scene);
        savesceneas.setText(scene.getName());
      }
      catch (Exception exp) {
        exp.printStackTrace();
      }
    }
    
    if (e.getSource()==elementlist) {
      if (elementlist.getSelectedIndex()<0 || e.getValueIsAdjusting()) return;
      try {
        scene.add(project.loadSceneElement(elementlist.getSelectedValue().toString()));
        scene.setSelectedIndex(0);
      }
      catch (Exception exp) {
        exp.printStackTrace();
      }
    }
  }
  
}
