package de.opalpiraten.scenepainter.gui;
import java.awt.*; 
import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;

/**
 * The Mainwindow of the Application
 */
public class Workspace extends JDesktopPane implements ActionListener {
  
  /**
   * The workspace is associated with exactly one Project
   */
  private Project project;
  
  /**
   * The scene currently showing in the Viewer
   */
  private Scene scene;
  
  /**
   * The Editor for Projects
   */
  protected ProjectEditor projecteditor = new ProjectEditor();
  
  /**
   * The editor for Scenes
   */
  protected SceneEditor sceneeditor = new SceneEditor();
  
  /**
   * The Frontend to the SceneElement editors
   */
  protected SceneElementEditor sceneelementeditor = new SceneElementEditor();
  
  /**
   * The Viewer
   */
  protected Viewer viewer = new Viewer();
  
  /**
   * The WorkspaceListeners
   */
  private LinkedList listeners = new LinkedList();
  
  /**
   * Sole constructor
   */
  public Workspace() {
    super();
    add(viewer,new Integer(-1));
    
    add(projecteditor);
    add(sceneelementeditor);
    add(sceneeditor);
    
    addWorkspaceListener(projecteditor);
    addWorkspaceListener(viewer);
    addWorkspaceListener(sceneelementeditor);
    addWorkspaceListener(sceneeditor);
    
    Menus.elements.setSelected(true);
    Menus.current.addActionListener(this);
    Menus.about.addActionListener(this);
    Menus.intro.addActionListener(this);
    Menus.notools.addActionListener(this);
    Menus.scene.addActionListener(this);
    Menus.project.addActionListener(this);
    Menus.elements.addActionListener(this);

    sceneelementeditor.setLocation(3,3);
    viewer.setLocation(sceneelementeditor.getWidth()+6,3);
    viewer.setSize(new Dimension(800,sceneelementeditor.getHeight()));
    sceneeditor.setLocation(sceneelementeditor.getWidth()+6,3);
    projecteditor.setLocation(sceneelementeditor.getWidth()+sceneeditor.getWidth()+9,3);
    sceneelementeditor.show();
    
    setDragMode(OUTLINE_DRAG_MODE);
  }
  
  /**
   * Add a Workspace listener to be notified when the workign set changes
   */
  public void addWorkspaceListener(WorkspaceListener l) {
    if (!listeners.contains(l)) listeners.add(l);
  }
  
  /**
   * Desubscribe a workspace listener
   */
  public void removeWorkspaceListener(WorkspaceListener l) {
    listeners.remove(l);
  }
  
  /**
   * Set the Project to be shown in the workspace and notify all editors
   * of the change.
   */
  public void setProject(Project p) {
    project=p;

    ListIterator it= listeners.listIterator(0);
    while(it.hasNext()) {
      ((WorkspaceListener)it.next()).replaceProject(p);
    } 
  }
  
  /**
   * Query the Project currently active
   */
  public Project getProject() {
    return project;
  }
  
  /**
   * Query the Scene currently being worked on
   */
  public Scene getScene() {
    return scene;
  }
  
  /**
   * Set Scene in the Workspace (and propagate it to any workspacelistener)
   */
  public void setScene(Scene s) {
    scene=s;
    ListIterator it= listeners.listIterator(0);
    while(it.hasNext()) {
      ((WorkspaceListener)it.next()).replaceScene(s);
    } 
  }
  
  /**
   * Get a hint where to place a frame
   * @param a frame about to be placed
   * @return a suggested spot of where to place the window
   */
  public Point getPlacementHint(JInternalFrame jr) {
    jr.pack(); // Just to be sure
    Point ret =  new Point(3,getHeight()-1-jr.getHeight());
    return ret;
  }

  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    
    if(src==Menus.project) {
      if (Menus.project.isSelected()) projecteditor.show();
      else projecteditor.setVisible(false);
    }
    
    if(src==Menus.scene) {
      if (Menus.scene.isSelected()) sceneeditor.show();
      else sceneeditor.setVisible(false);
    }
    
    if(src==Menus.elements) {
      if (Menus.elements.isSelected()) sceneelementeditor.show();
      else sceneelementeditor.setVisible(false);
    }
    
    if (src==Menus.current) {
      HelpBrowser h = new HelpBrowser(getSelectedFrame());
      h.setBounds(0,0,640,480);
      add(h);
      h.show();
    }
    
    if (src==Menus.license) {
      HelpBrowser h = new HelpBrowser(new HelpLicense());
      h.setBounds(0,0,640,480);
      add(h);
      h.show();
    }
    
    if (src==Menus.about) {
      HelpBrowser h = new HelpBrowser(new HelpAbout());
      h.setBounds(0,0,640,480);
      add(h);
      h.show();
    }
    
    if (src==Menus.intro) {
      HelpBrowser h = new HelpBrowser(new HelpIntro());
      h.setBounds(0,0,640,480);
      add(h);
      h.show();
    }
    
    if (src==Menus.notools) {
      HelpBrowser h = new HelpBrowser(new HelpNoTools());
      h.setBounds(0,0,640,480);
      add(h);
      h.show();
    }
  }
  
}